# -*- coding: UTF-8 -*-
"""

tests du langage de templating créole
ce langage est hérité de Cheetah

"""
import os.path as osp
from os import remove
from py.test import raises
from creole.error import TemplateError
from creole.cfgparser import EoleDict

HERE = osp.join(osp.dirname(__file__))
DATADIR = osp.join(HERE, 'data')

GENERATED_FILES = set()

def check_filecontent(got, expected):
    """utilitaire de comparaison de 2 fichiers"""
    expected_content = file(expected).read().strip()
    got_content = file(got).read().strip()
    assert got_content == expected_content


def test_language_constructions():
    """tests des constructions de base du langage"""
    testnames = ('testif', 'testfor', 'testimbrication', 'testdefined', )
    for testname in testnames:
        resultfile = osp.join(DATADIR, '%s.result' % testname)
        eoledict = EoleDict(templatedir=DATADIR)
        eoledict.read(osp.join(DATADIR, 'eole.xml'))
        eoledict.instance()
        generated = '%s.output' % testname
        GENERATED_FILES.add(generated)
        yield check_filecontent, generated, resultfile


def test_mastervar_nomulti():
    """chargement d'une variable eole de type simple
    mais ayant des variables 'esclave' (variable maitre)"""
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'masternomulti.xml'))
    ip = eoledict.get_final_value('ip')
    
    assert ip.value == '172.17.1.4'
    assert ip.network == '172.17.0.0'
    assert ip.netmask == '255.255.0.0'
        
def test_multivar_iter():
    """chargement d'une variable eole de type multiple
    test en convertissant en liste
    """
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'multi.xml'))
    iplist = list(eoledict.variables['iplist'])
    assert iplist == ['172.17.1.4', '172.17.1.5', '172.17.1.6']


def test_multivar_index():
    """utilisation de la notation indicée
    pour l'accès aux variables de type 'multi'
    """
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'multi.xml'))
    iplist = eoledict.variables['iplist']
    assert iplist[0] == '172.17.1.4'
    assert iplist[1] == '172.17.1.5'
    assert iplist[2] == '172.17.1.6'
    

def test_multi_no_ini():
    """tests pour les variables multi-valuées (sans ini)
    """
    resultfile = osp.join(DATADIR, 'testmulti.result')
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'multi.xml'))
    eoledict.instance()    
    GENERATED_FILES.add('testmulti.output')
    check_filecontent('testmulti.output', resultfile)


def test_multi_with_ini():
    """tests pour les variables multi-valuées (avec ini)
    """
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'multi.xml'))
    eoledict.load_values(osp.join(DATADIR, 'multi.eol'))
    eoledict.instance()
    resultfile = osp.join(DATADIR, 'testmulti2.result')
    GENERATED_FILES.add('testmulti.output')
    check_filecontent('testmulti.output', resultfile)

def test_multislave():
    """tests pour les variables multi-valuées avec esclaves"""
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'multislave.xml'))
    eoledict.load_values(osp.join(DATADIR, 'multislave.eol'))
    eoledict.instance()
    resultfile = osp.join(DATADIR, 'testmultislave.result')
    GENERATED_FILES.add('testmultislave.output')
    check_filecontent('testmultislave.output', resultfile)
    
def test_undefined():
    """tests pour les templates dont une variable n'existe pas"""
    eoledict = EoleDict(templatedir=DATADIR)
    eoledict.read(osp.join(DATADIR, 'missing.xml'))
    assert raises(TemplateError, eoledict.instance)

def test_builtins():
    """la fonctions interne disponible is_defined()
    - dans le cas où elle n'est pas définie
    - dans le cas où elle est accessible (par un dictionnaire additionnel)
    """
    testnames = ('testbuiltins', 'testbuiltins2', )
    for testname in testnames:
        resultfile = osp.join(DATADIR, '%s.result' % testname)
        eoledict = EoleDict(templatedir=DATADIR)
        eoledict.read(osp.join(DATADIR, 'eole.xml'))
        eoledict.read(osp.join(DATADIR, 'eole2.xml'))
        eoledict.instance()
        generated = '%s.output' % testname
        GENERATED_FILES.add(generated)
        yield check_filecontent, generated, resultfile

def test_eos_builtins():
    """teste les fonctions eos disponibles dans *eoscheetah*
    comme **list** ou les conversions de masque
    """
    testnames = ('eoscheetah',)
    for testname in testnames:
        resultfile = osp.join(DATADIR, '%s.result' % testname)
        eoledict = EoleDict(templatedir=DATADIR)
        eoledict.read(osp.join(DATADIR, 'eos_builtins.xml'))
        eoledict.instance()
        generated = '%s.output' % testname
        GENERATED_FILES.add(generated)
        yield check_filecontent, generated, resultfile


def test_comments():
    """Teste le traitement des commentaires (#) dans un template cible
    """
    testnames = ('comment1', )
    for testname in testnames:
        resultfile = osp.join(DATADIR, '%s.result' % testname)

        eoledict = EoleDict(templatedir=DATADIR)
        eoledict.read(osp.join(DATADIR, 'comment.xml'))
        eoledict.instance()
        generated = '%s.output' % testname
        GENERATED_FILES.add(generated)

        yield check_filecontent, generated, resultfile

def teardown_module(module):
    "enleve des fichiers generes par le templating"
    for generated in GENERATED_FILES:
        remove(generated)
